//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Net;
using System.Runtime.Remoting.Lifetime;
using System.Security.Permissions;
using Bon.Remoting;

namespace Bon
{
	/// <summary>
	/// `[i[̎NXłB
	/// </summary>
	internal sealed class TunerFacade : MarshalByRefObject, IRemoteTuner
	{
		private readonly ITuner tuner_;
		private readonly Uri[] streamUrls_;
		private readonly Dictionary<string, string> defaultPlayers_;
		private readonly Dictionary<IPEndPoint, IDataQueue> queues_ = new Dictionary<IPEndPoint, IDataQueue>();

		public TunerFacade(ITuner tuner, string[] addresses, int port, string path,
			Dictionary<string, string> defaultPlayers)
		{
			tuner_ = tuner;
			defaultPlayers_ = defaultPlayers;
			streamUrls_ = new Uri[addresses.Length];
			for (int i = 0; i < streamUrls_.Length; i++)
			{
				string url = string.Format("http://{0}:{1}{2}", addresses[i], port, path);
				streamUrls_[i] = new Uri(url);
			}
		}

		[SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.Infrastructure)]
		public override object InitializeLifetimeService()
		{
			ILease lease = (ILease)base.InitializeLifetimeService();
			if (lease.CurrentState == LeaseState.Initial)
			{
				lease.InitialLeaseTime = TimeSpan.Zero;
			}
			return lease;
		}

		internal void AddEndPoint(IPEndPoint endPoint, IDataQueue queue)
		{
			lock (queues_)
			{
				queues_.Add(endPoint, queue);
			}
		}

		internal void RemoveEndPoint(IPEndPoint endPoint)
		{
			lock (queues_)
			{
				queues_.Remove(endPoint);
			}
		}

		public IRemoteTuningSpaceInformation[] GetTuningSpaces()
		{
			ITuningSpaceInformation[] tuningSpaces = tuner_.TuningSpaces;
			IRemoteTuningSpaceInformation[] result = new IRemoteTuningSpaceInformation[tuningSpaces.Length];
			for (int i = 0; i < result.Length; i++)
			{
				result[i] = new RemoteTuningSpaceInformation(tuningSpaces[i]);
			}
			return result;
		}

		public string GetDefaultPlayerPath(string architectureName)
		{
			return defaultPlayers_[architectureName];
		}

		public IRemoteTunerStatus GetTunerStatus()
		{
			return new RemoteTunerStatus(tuner_, streamUrls_, new List<IPEndPoint>(queues_.Keys).ToArray());
		}

		public IRemoteProgramInformation[] GetPrograms()
		{
			IProgramInformation[] programs = tuner_.Programs;
			IRemoteProgramInformation[] result = new IRemoteProgramInformation[programs.Length];
			for (int i = 0; i < result.Length; i++)
			{
				result[i] = new RemoteProgramInformation(programs[i]);
			}
			return result;
		}

		public void SetEmmProcess(bool enable)
		{
			lock (queues_)
			{
				tuner_.EnableEmmProcess = enable;
			}
		}

		public void SetLnbPower(bool enable)
		{
			lock (queues_)
			{
				tuner_.EnableLnbPower = enable;
			}
		}

		public void SetChannel(int tuningSpace, int channel, int programID)
		{
			lock (queues_)
			{
				tuner_.SetChannel(tuningSpace, channel, programID);
			}
		}

		public int GetProgramID(IPEndPoint endPoint)
		{
			try
			{
				lock (queues_)
				{
					return queues_[endPoint].ProgramID;
				}
			}
			catch (Exception ex)
			{
				throw new BonException(
					ex, "Cannot get program ID: endPoint={0}", endPoint);
			}
		}

		public void SetProgramID(IPEndPoint endPoint, int programID)
		{
			try
			{
				lock (queues_)
				{
					queues_[endPoint].ProgramID = programID;
				}
			}
			catch (Exception ex)
			{
				throw new BonException(
					ex, "Cannot set program ID: endPoint={0}, programID={1}",
					endPoint, programID);
			}
		}

		public void DisconnectStream(IPEndPoint endPoint)
		{
			try
			{
				lock (queues_)
				{
					queues_[endPoint].Dispose();
				}
			}
			catch
			{
			}
		}
	}
}
